<?php
/**
 * Plugin plxGoaccess
 **/
class plxOpenStreetMap extends plxPlugin {
	public $plugName;
	public function __construct($default_lang) {

		# appel du constructeur de la classe plxPlugin (obligatoire)admin
		parent::__construct($default_lang);
		$this->plugName = $this->plug['name'];

		# droits pour accéder à la page config.php du plugin
		$this->setConfigProfil(PROFIL_ADMIN);
		# droits pour accéder à la page admin.php du plugin
		$this->setAdminProfil(PROFIL_ADMIN ); // admin et gestionnaire
		# initialisation des variables de la classe
	}
	public function OnActivate() {
        # code à exécuter à l’activation du plugin
		$file_plugOnactivateConfig = (PLX_PLUGINS.$this->plugName.'/OnactivateConfig.xml');
		$file_plugConfig = (PLX_ROOT.PLX_CONFIG_PATH.'plugins/'.$this->plugName.'.xml');
		if (!file_exists(realpath($file_plugConfig)))
		{
			copy ($file_plugOnactivateConfig, $file_plugConfig); // init de la configuration par defaut
        }
	}
	/**
	 * Méthode qui crée le fichier .htaccess en cas de réécriture d'urls
	 * Par défaut les valeurs sont prises dans la config du plugin pour la carte demandée
	 * @param	map	numéro de la carte 
	 * @param	data_center		latitude, longitude du centre de la carte, avec point décimal et séparée par une virgule:
	 * @param	data_zoom		niveaux de zoom : à l"init, mini, maxi : 3 nb entiers (de 1 à 20) séparés par une virgule::
	 * @param	maxClusterRadius	rayon du cluster : 1 à 80; plus grand est ce nombre, plus les marqueurs seront groupés::
	 * @param	gps_points		la liste des marqueurs sur la carte (cf page admin)
	 * @param	zones		POLYGONE ou trace gpx SUR LA CARTE (cf page admin)
	 * @param	map_id		latitude, longitude du centre de la carte, avec point décimal et séparée par une virgule:
	 * @param	icon_height	Hauteur de l"icone (la largeur et la position de l"ancrage sont calculées automatiquement)
	 * @return	le code html et js 
	 * @author	JP DOUILLARD et d'autres contributeurs 
	 **/	
	public function echomap($map=0, $data_center="", $data_zoom="", $maxClusterRadius=35, $gps_points="", $zones="", $map_id = "", $icon_height="") {
$lats = array();
$longs = array();
$infos = array();
$icons = array();
$separator = "/\r\n|\r|\n|\|/";
$output = "";
$url = "";
if ($gps_points == "") $gps_points = $this->getParam($map . '_points');

if ($gps_points != "" && str_contains($gps_points, '.gpx')) { // il y a un parametre de fichier trace gpx
	$gps_points = preg_replace($separator, "|", $gps_points);
	$gps_points_array = explode('|', $gps_points);
	$gps_points = "";
	$sep = "";
	foreach ($gps_points_array as $data => $value) {	
		if (str_contains($value, '.gpx')) {$url = $value;}
		else {
			$gps_points .= $sep . $value;
			$sep = "|";
		};
	}
};
if ($gps_points != "") {
	$gps_points = preg_replace($separator, ",", $gps_points);
	$gps_points = explode(',', $gps_points);
	$gps_points = array_chunk($gps_points, 4); // 
	$lats = array_column($gps_points, 0);
	$longs = array_column($gps_points, 1);
	$infos = array_column($gps_points, 2);
	$icons = array_column($gps_points, 3);
	}
else $gps_points = array();

if ($zones === "") $zones = $this->getParam($map . '_zone');
if ($zones != "" && str_contains($zones, '.gpx')) { // il y a un parametre de fichier trace gpx
	$zones = preg_replace($separator, "|", $zones);
	$zones_array = explode('|', $zones);
	$zones = "";
	$sep = "";
	foreach ($zones_array as $data => $value) {	
		if (str_contains($value, '.gpx')) {$url = $value;}
		else {
			$zones .= $sep . $value;
			$sep = "|";
		};
	}
};
if ($zones != "") {
$zones = preg_replace($separator, ",", $zones);
$zones = explode(',', $zones);
$zones = array_chunk($zones, 2);
}
else $zones = array();
$maxClusterRadius = $this->getParam($map . '_cluster_radius');
if ($icon_height === "") $icon_height = $this->getParam($map . '_icon_height');
if ($icon_height == "") {
$icon_height = 41;
};	
if ($data_zoom === "") $data_zoom = $this->getParam($map . '_data_zoom');
$data_zoom = preg_split("/[\r,]+/", $data_zoom);
if ($data_center === "") $data_center = $this->getParam($map . '_data_center');
$data_center = preg_split("/[\r,]+/", $data_center);
$output = "";
$output .= ('
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.7.1/leaflet.css" />
<link rel="stylesheet" type="text/css" href="https://unpkg.com/leaflet.markercluster@1.3.0/dist/MarkerCluster.css" />
<link rel="stylesheet" type="text/css" href="https://unpkg.com/leaflet.markercluster@1.3.0/dist/MarkerCluster.Default.css" />	
<style>
/***** map *****/
      /* Always set the map height explicitly to define the size of the div
       * element that contains the map. */
#map'.$map_id .' {
        height: 450px;
		width: 100%;
		position: relative;
		z-index: 0;
}
</style>
		<div id="map'.$map_id .'" class="map'.$map.'"></div>
       <script src="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.7.1/leaflet.js"></script>
        <script type="text/javascript" src="https://unpkg.com/leaflet.markercluster@1.4.1/dist/leaflet.markercluster.js"></script>
		<script src="https://unpkg.com/leaflet-gpx@1.7.0/gpx.js"></script>
<script type="text/javascript">
');
$js_array = json_encode($data_zoom);
$output .=  "let data_zoom$map_id = ". $js_array . ";\n";
$js_array = json_encode($data_center);
$output .=  "let data_center$map_id = ". $js_array . ";\n";
$js_array = json_encode($zones);
$output .=  "let points$map_id = ". $js_array . ";\n";
$js_array = $maxClusterRadius;
$output .=  "let maxClusterRadius$map_id = ". $js_array . ";\n";
$js_array = $icon_height;
$output .=  "let icon_height$map_id = ". $js_array . ";\n";
$js_array = json_encode($lats);
$output .=  "let lats$map_id = ". $js_array . ";";
$js_array = json_encode($longs);
$output .=  "let longs$map_id = ". $js_array . ";";
$js_array = json_encode($infos);
$output .=  "let ips$map_id = ". $js_array . ";";
$js_array = json_encode($icons);
$output .=  "let icons$map_id = ". $js_array . ";";
$js_array = $url;
$output .=  "let url_gpx$map_id = '". $js_array . "';\n";
$output .= ('
	    var lat'.$map_id .' = data_center'.$map_id .'[0];
	    var lon'.$map_id .' = data_center'.$map_id .'[1];
	    var macarte = null;
	    // Fonction d"initialisation de la carte
function initMap'.$map_id .'() {
	macarte = L.map("map'.$map_id .'").setView([lat'.$map_id .', lon'.$map_id .'], data_zoom'.$map_id .'[0]);
	// Tableau de points
let options = {
    color: "rgba(61,171,158,1)",
    fillColor: "rgba(61,171,158,1)",
    dashArray: "15,2,10"
};
let prefX = "../../";
var control = L.control.layers(null, null).addTo(macarte);

new L.GPX(url_gpx'.$map_id .', {
        async: true,
        marker_options: {
          startIconUrl: prefX+"plugins/plxOpenStreetMap/icon/pin-icon-start.png",
          endIconUrl:   prefX+"plugins/plxOpenStreetMap/icon/pin-icon-end.png",
          shadowUrl:    prefX+"plugins/plxOpenStreetMap/icon/pin-shadow.png",
          wptIconUrls:  prefX+"plugins/plxOpenStreetMap/icon/pin-icon-wpt.png",
			iconSize: [, icon_height'.$map_id .'],
			iconAnchor: [iconAnchor_left(prefX+"data/medias/fanion-loriginal.svg", icon_height'.$map_id .'), icon_height'.$map_id .'],
			popupAnchor: [-3, -10],
			shadowSize: [61, icon_height'.$map_id .']
        },
      }).on("loaded", function(e) {
        var gpx = e.target;
		macarte.fitBounds(gpx.getBounds(), {padding: [100,10]});
		console.log (macarte.getCenter());
//		macarte.setView(macarte.getCenter(gpx.getBounds()), 12);
        control.addOverlay(gpx, gpx.get_name());
      }).addTo(macarte);
L.polygon(points'.$map_id .', options).addTo(macarte);
	// Leaflet ne récupère pas les cartes (tiles) sur un serveur par défaut. Nous devons lui préciser où nous souhaitons les récupérer. Ici, openstreetmap.fr
	L.tileLayer("https://{s}.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png", {
		// Il est toujours bien de laisser le lien vers la source des données
		attribution: "données © OpenStreetMap/ODbL - rendu OSM France",
		minZoom: data_zoom'.$map_id .'[1],
		maxZoom: data_zoom'.$map_id .'[2]
	}).addTo(macarte);
	var markersCluster = new L.MarkerClusterGroup({
		maxClusterRadius : maxClusterRadius'.$map_id .'	
});
	var icon_url = ""; // le dossier doit être préciser dans les paramètres ; pas de dossier par defaut
	for (var i = 0; i < lats'.$map_id .'.length; i++) {
		var icon = icons'.$map_id .'[i].trim();
		if ((icon == "blue") || (icon == "gold") || (icon == "red")|| (icon == "green")|| (icon == "orange")|| (icon == "yellow")|| (icon == "violet")|| (icon == "grey")|| (icon == "black"))
		{
		var myIcon = new L.Icon({
			iconUrl: "https://raw.githubusercontent.com/pointhi/leaflet-color-markers/master/img/marker-icon-2x-" + icon +".png",
			shadowUrl: "https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/images/marker-shadow.png",
			iconSize: [, icon_height'.$map_id .'],
			iconAnchor: [iconAnchor_left("https://raw.githubusercontent.com/pointhi/leaflet-color-markers/master/img/marker-icon-2x-" + icon +".png", icon_height'.$map_id .'), icon_height'.$map_id .'],
			popupAnchor: [1, -34],
			shadowSize: [, icon_height'.$map_id .']
		});
		}
		else {
		if (icon != "none") {
		var myIcon = new L.icon({
			iconUrl: icon_url + icon,
			shadowUrl: prefX+"data/medias/fanion-loriginal-shawdow.png",
			iconSize: [, icon_height'.$map_id .'],
			iconAnchor: [iconAnchor_left(icon_url + icon, icon_height'.$map_id .'), icon_height'.$map_id .'],
			popupAnchor: [-3, -10],
			shadowSize: [, icon_height'.$map_id .']
		});
		}
		};
		var latLng = new L.LatLng(lats'.$map_id .'[i], longs'.$map_id .'[i]);

		if (ips'.$map_id .'[i] == ""){
			if (icon == "none") {
			var marker = new L.Marker(latLng);
			}
			else
			{
			var marker = new L.Marker(latLng, {icon: myIcon});
			};
		}
		else {
			if (icon == "none") {
			var marker = new L.Marker(latLng).bindPopup(ips'.$map_id .'[i]);
			}
			else
			{
			var marker = new L.Marker(latLng, {icon: myIcon}).bindPopup(ips'.$map_id .'[i]);
			};
			};
		markersCluster.addLayer(marker);
	}
		
		
	macarte.addLayer(markersCluster);
}
function iconAnchor_left(url, width) {
const img = new Image();
img.onload = function() {
}
img.src = url;
iconSize_ratio = img.width / img.height;
icon_height_real = width;
icon_width_real = icon_height_real * iconSize_ratio ;
return icon_width_real / 2;
};
		
$(document).ready(function() {
		// Fonction d"initialisation qui s"exécute lorsque le DOM est chargé
		initMap'.$map_id .'(); 
});
</script> 
');
return $output;
}

}
?>